@@ -140,13 +140,11 @@ static void proxy_gc_timerpoke(evutil_socket_t fd, short event, void *arg) {
 
 bool proxy_bufmem_checkadd(LIBEVENT_THREAD *t, int len) {
     bool oom = false;
-    pthread_mutex_lock(&t->proxy_limit_lock);
     if (t->proxy_buffer_memory_used > t->proxy_buffer_memory_limit) {
         oom = true;
     } else {
         t->proxy_buffer_memory_used += len;
     }
-    pthread_mutex_unlock(&t->proxy_limit_lock);
     return oom;
 }
 
@@ -208,9 +206,7 @@ void process_proxy_stats(void *arg, ADD_STAT add_stats, void *c) {
             }
         }
         WSTAT_UL(t);
-        pthread_mutex_lock(&t->proxy_limit_lock);
         buffer_memory_used += t->proxy_buffer_memory_used;
-        pthread_mutex_unlock(&t->proxy_limit_lock);
     }
 
     // return all of the user generated stats
@@ -439,7 +435,6 @@ void proxy_thread_init(void *ctx, LIBEVENT_THREAD *thr) {
         fprintf(stderr, "Failed to allocate proxy thread stats\n");
         exit(EXIT_FAILURE);
     }
-    pthread_mutex_init(&thr->proxy_limit_lock, NULL);
     thr->proxy_ctx = ctx;
 
     // Initialize the lua state.
@@ -708,9 +703,7 @@ void complete_nread_proxy(conn *c) {
     rq->pr.vbuf = c->item;
     c->item = NULL;
     c->item_malloced = false;
-    pthread_mutex_lock(&thr->proxy_limit_lock);
     thr->proxy_buffer_memory_used += rq->pr.vlen;
-    pthread_mutex_unlock(&thr->proxy_limit_lock);
 
     conn_resp_suspend(rctx->c, rctx->resp);
     proxy_run_rcontext(rctx);
@@ -762,29 +755,21 @@ void proxy_out_errstring(mc_resp *resp, char *type, const char *str) {
 // noreply mode from the response object is that the proxy can return strings
 // manually, so we have no way to obey what the original request wanted in
 // that case.
-static void _set_noreply_mode(mc_resp *resp, mcp_resp_t *r) {
-    switch (r->mode) {
-        case RESP_MODE_NORMAL:
-            break;
-        case RESP_MODE_NOREPLY:
-            // ascii noreply only threw egregious errors to client
-            if (r->status == MCMC_OK) {
-                resp->skip = true;
-            }
-            break;
-        case RESP_MODE_METAQUIET:
-            if (r->resp.code == MCMC_CODE_END) {
-                resp->skip = true;
-            } else if (r->cmd != CMD_MG && r->resp.code == MCMC_CODE_OK) {
-                // FIXME (v2): mcmc's parser needs to help us out a bit more
-                // here.
-                // This is a broken case in the protocol though; quiet mode
-                // ignores HD for mutations but not get.
-                resp->skip = true;
-            }
-            break;
-        default:
-            assert(1 == 0);
+static void _set_noreply_mode(mc_resp *resp, mcp_request_t *rq, mcp_resp_t *r) {
+    if (rq->pr.cmd_type == CMD_TYPE_META
+        && mcmc_token_has_flag_bit(&rq->pr.tok, (uint64_t)1<<48) == MCMC_OK) {
+        if (r->resp.code == MCMC_CODE_END) {
+            resp->skip = true;
+        } else if (r->cmd != CMD_MG && r->resp.code == MCMC_CODE_OK) {
+            // FIXME (v2): mcmc's parser needs to help us out a bit more
+            // by maybe flagging a response as quietable.
+            // This is a broken case in the protocol though; quiet mode
+            // ignores HD for mutations but not get.
+            resp->skip = true;
+        }
+    } else if (rq->pr.noreply && r->status == MCMC_OK) {
+        // ascii noreply only threw egregious errors to client
+        resp->skip = true;
     }
 }
 
@@ -845,7 +830,7 @@ int proxy_run_rcontext(mcp_rcontext_t *rctx) {
             mcp_resp_t *r = NULL;
             P_DEBUG("%s: coroutine completed. return type: %d\n", __func__, type);
             if (type == LUA_TUSERDATA && (r = luaL_testudata(Lc, 1, "mcp.response")) != NULL) {
-                _set_noreply_mode(resp, r);
+                _set_noreply_mode(resp, rctx->request, r);
                 if (r->status != MCMC_OK && r->resp.type != MCMC_RESP_ERRMSG) {
                     proxy_out_errstring(resp, PROXY_SERVER_ERROR, "backend failure");
                 } else if (r->cresp) {
@@ -865,7 +850,7 @@ int proxy_run_rcontext(mcp_rcontext_t *rctx) {
                     resp->write_and_free = r->buf;
                     resp_add_iov(resp, r->buf, r->blen);
                     // stash the length to later remove from memory tracking
-                    resp->wbytes = r->blen + r->extra;
+                    resp->wbytes = r->blen;
                     resp->proxy_res = true;
                     r->buf = NULL;
                 } else {
@@ -883,6 +868,21 @@ int proxy_run_rcontext(mcp_rcontext_t *rctx) {
                 proxy_out_errstring(resp, PROXY_SERVER_ERROR, "bad response");
             }
 
+            if (rctx->ascii_multiget && !resp->skip) {
+                struct iovec *iov = &resp->iov[resp->iovcnt-1];
+                // look for END at end of final iov and blank it.
+                // Bad hack, but centralized here now.
+                if (iov->iov_len >= ENDLEN &&
+                        memcmp(((char *)iov->iov_base)+(iov->iov_len-ENDLEN), ENDSTR, ENDLEN) == 0) {
+                    iov->iov_len -= ENDLEN;
+                    resp->tosend -= ENDLEN;
+                    if (resp->tosend == 0) {
+                        // some platforms don't like 0 byte iov's.
+                        // may be misremembering, double check? mac os?
+                        resp->skip = true;
+                    }
+                }
+            }
             conn_resp_unsuspend(rctx->c, resp);
             rctx->c = NULL; // *conn cannot be used past this point!
             rctx->pending_reqs--;
@@ -938,6 +938,10 @@ int proxy_run_rcontext(mcp_rcontext_t *rctx) {
         // result object normally.
         P_DEBUG("%s: Failed to run coroutine: %s\n", __func__, lua_tostring(Lc, -1));
         LOGGER_LOG(NULL, LOG_PROXYEVENTS, LOGGER_PROXY_ERROR, NULL, lua_tostring(Lc, -1));
+        if (settings.verbose) {
+            luaL_traceback(Lc, Lc, lua_tostring(Lc, -1), 1);
+            fprintf(stderr, "proxy_lua_error: %s\n", lua_tostring(Lc, -1));
+        }
         if (!rctx->parent) {
             proxy_out_errstring(resp, PROXY_SERVER_ERROR, "lua failure");
             conn_resp_unsuspend(rctx->c, resp);
@@ -1003,21 +1007,9 @@ static void proxy_process_command(conn *c, char *command, size_t cmdlen, bool mu
     }
 
     if (!hook_ref.lua_ref) {
-        // need to pass our command string into the internal handler.
-        // to minimize the code change, this means allowing it to tokenize the
-        // full command. The proxy's indirect parser should be built out to
-        // become common code for both proxy and ascii handlers.
-        // For now this means we have to null-terminate the command string,
-        // then call into text protocol handler.
-        // FIXME (v2): use a ptr or something; don't like this code.
-        if (cmdlen > 1 && command[cmdlen-2] == '\r') {
-            command[cmdlen-2] = '\0';
-        } else {
-            command[cmdlen-1] = '\0';
-        }
         // lets nread_proxy know we're in ascii mode.
         c->proxy_rctx = NULL;
-        process_command_ascii(c, command);
+        process_command_ascii(c, command, cmdlen);
         return;
     }
 
@@ -1027,21 +1019,38 @@ static void proxy_process_command(conn *c, char *command, size_t cmdlen, bool mu
     // might be better to split this function; the below bits turn into a
     // function call, then we don't re-process the above bits in the same way?
     // The way this is detected/passed on is very fragile.
-    if (!multiget && pr.cmd_type == CMD_TYPE_GET && pr.has_space) {
-        uint32_t keyoff = pr.tokens[pr.keytoken];
-        while (pr.klen != 0) {
+    if (!multiget && pr.cmd_type == CMD_TYPE_GET && pr.tok.ntokens > pr.keytoken+1) {
+        uint32_t keyoff = pr.tok.tokens[pr.keytoken];
+        const char *curkey = pr.request + keyoff;
+        int klen = pr.klen;
+        int cmd_prefix_len = pr.tok.tokens[pr.keytoken];
+        size_t endlen = 0;
+
+        // TODO: TEST THIS CASE
+        // check once if our prefix is too long.
+        if (cmd_prefix_len > MAX_CMD_PREFIX) {
+            if (!resp_start(c)) {
+                conn_set_state(c, conn_closing);
+                return;
+            }
+            proxy_out_errstring(c->resp, PROXY_CLIENT_ERROR, "malformed request");
+            return;
+        }
+
+        assert(pr.request[pr.reqlen-1] == '\n');
+        if (pr.request[pr.reqlen-2] == '\r') {
+            endlen = pr.reqlen - 2;
+        } else {
+            endlen = pr.reqlen - 1;
+        }
+
+        while (klen != 0) {
             char temp[KEY_MAX_LENGTH + MAX_CMD_PREFIX + 30];
             char *cur = temp;
             // Core daemon can abort the entire command if one key is bad, but
             // we cannot from the proxy. Instead we have to inject errors into
             // the stream. This should, thankfully, be rare at least.
-            if (pr.tokens[pr.keytoken] > MAX_CMD_PREFIX) {
-                if (!resp_start(c)) {
-                    conn_set_state(c, conn_closing);
-                    return;
-                }
-                proxy_out_errstring(c->resp, PROXY_CLIENT_ERROR, "malformed request");
-            } else if (pr.klen > KEY_MAX_LENGTH) {
+            if (klen > KEY_MAX_LENGTH) {
                 if (!resp_start(c)) {
                     conn_set_state(c, conn_closing);
                     return;
@@ -1049,12 +1058,12 @@ static void proxy_process_command(conn *c, char *command, size_t cmdlen, bool mu
                 proxy_out_errstring(c->resp, PROXY_CLIENT_ERROR, "key too long");
             } else {
                 // copy original request up until the original key token.
-                memcpy(cur, pr.request, pr.tokens[pr.keytoken]);
-                cur += pr.tokens[pr.keytoken];
+                memcpy(cur, pr.request, cmd_prefix_len);
+                cur += cmd_prefix_len;
 
                 // now copy in our "current" key.
-                memcpy(cur, &pr.request[keyoff], pr.klen);
-                cur += pr.klen;
+                memcpy(cur, curkey, klen);
+                cur += klen;
 
                 memcpy(cur, "\r\n", 2);
                 cur += 2;
@@ -1064,8 +1073,31 @@ static void proxy_process_command(conn *c, char *command, size_t cmdlen, bool mu
                 proxy_process_command(c, temp, cur - temp, PROCESS_MULTIGET);
             }
 
-            // now advance to the next key.
-            keyoff = _process_request_next_key(&pr);
+            // FIXME: test multigets with excess space at the end.
+            // Find the next key manually. This keeps the special cased code
+            // here instead of adding extra junk to the parser structure.
+            curkey += klen;
+            int remain = endlen - (curkey - pr.request);
+            while (remain) {
+                if (*curkey == ' ') {
+                    remain--;
+                    curkey++;
+                } else {
+                    break;
+                }
+            }
+
+            if (remain) {
+                const char *s = memchr(curkey, ' ', remain);
+                if (s != NULL) {
+                    klen = s - curkey;
+                } else {
+                    klen = remain;
+                }
+            } else {
+                // no more keys
+                break;
+            }
         }
 
         if (!resp_start(c)) {
@@ -1133,7 +1165,7 @@ static void proxy_process_command(conn *c, char *command, size_t cmdlen, bool mu
     }
 
     mcp_set_request(&pr, rctx->request, command, cmdlen);
-    rctx->request->ascii_multiget = multiget;
+    rctx->ascii_multiget = multiget;
     rctx->c = c;
     rctx->conn_fd = c->sfd;
     rctx->pending_reqs++; // seed counter with the "main" request
@@ -1211,20 +1243,19 @@ void mcp_set_resobj(mcp_resp_t *r, mcp_request_t *rq, mcp_backend_t *be, LIBEVEN
     // the coroutine but the structure doesn't allow that yet.
     // Should also be able to settle this exact mode from the parser so we
     // don't have to re-branch here.
-    if (rq->pr.noreply) {
-        if (rq->pr.cmd_type == CMD_TYPE_META) {
-            r->mode = RESP_MODE_METAQUIET;
-            for (int x = 2; x < rq->pr.ntokens; x++) {
-                if (rq->request[rq->pr.tokens[x]] == 'q') {
-                    rq->request[rq->pr.tokens[x]] = ' ';
+    if (rq->pr.cmd_type == CMD_TYPE_META) {
+        if (mcmc_token_has_flag_bit(&rq->pr.tok, (uint64_t)1<<48) == MCMC_OK) {
+            // FIXME: should this use the new interface? re-evaluate?
+            for (int x = 2; x < rq->pr.tok.ntokens; x++) {
+                if (rq->request[rq->pr.tok.tokens[x]] == 'q') {
+                    rq->request[rq->pr.tok.tokens[x]] = ' ';
                 }
             }
-        } else {
-            r->mode = RESP_MODE_NOREPLY;
-            rq->request[rq->pr.reqlen - 3] = 'Y';
         }
     } else {
-        r->mode = RESP_MODE_NORMAL;
+        if (rq->pr.noreply) {
+            rq->request[rq->pr.reqlen - 3] = 'Y';
+        }
     }
 
     r->cmd = rq->pr.command;
@@ -1271,7 +1302,6 @@ io_pending_proxy_t *mcp_queue_rctx_io(mcp_rcontext_t *rctx, mcp_request_t *rq, m
     p->rctx = rctx;
 
     if (rq) {
-        p->ascii_multiget = rq->ascii_multiget;
         // The direct backend object. Lc is holding the reference in the stack
         p->backend = be;
 
