@@ -1,9 +1,10 @@
 #ifndef PROTO_PROXY_H
 #define PROTO_PROXY_H
 
-void proxy_stats(void *arg, ADD_STAT add_stats, conn *c);
-void process_proxy_stats(void *arg, ADD_STAT add_stats, conn *c);
-void process_proxy_funcstats(void *arg, ADD_STAT add_stats, conn *c);
+void proxy_stats(void *arg, ADD_STAT add_stats, void *c);
+void process_proxy_stats(void *arg, ADD_STAT add_stats, void *c);
+void process_proxy_funcstats(void *arg, ADD_STAT add_stats, void *c);
+void process_proxy_bestats(void *arg, ADD_STAT add_stats, void *c);
 
 /* proxy mode handlers */
 int try_read_command_proxy(conn *c);
@@ -17,6 +18,7 @@ void proxy_start_reload(void *arg);
 int proxy_first_confload(void *arg);
 int proxy_load_config(void *arg);
 void proxy_worker_reload(void *arg, LIBEVENT_THREAD *thr);
+void proxy_gc_poke(LIBEVENT_THREAD *t);
 
 void proxy_submit_cb(io_queue_t *q);
 void proxy_complete_cb(io_queue_t *q);
